﻿using System;
using System.ComponentModel;
using System.Linq;
using System.Xml.Serialization;
using Microsoft.WowAddonStudio.FrameXml.Serialization.Attributes;
using Microsoft.WowAddonStudio.FrameXml.Serialization.Enumerations;

namespace Microsoft.WowAddonStudio.FrameXml.Serialization.LayoutFrames
{
    [Serializable]
    [DesignerCategory("code")]
    [XmlType(Namespace = "http://www.blizzard.com/wow/ui/")]
    [XmlRoot(Namespace = "http://www.blizzard.com/wow/ui/", IsNullable = false)]
    public class StatusBar : Frame
    {
        /// <summary>
        /// Gets or sets the draw layer to be used by the status bar. The default value is artwork.
        /// </summary>
        [XmlAttribute("drawLayer")]
        [Category("Behavior")]
        [DefaultValue(DrawLayer.Artwork)]
        [Description("Defines the draw layer to be used by the status bar. The default value is artwork.")]
        public DrawLayer DrawLayer
        {
            get { return Properties.GetValue<DrawLayer>("DrawLayer"); }
            set { Properties.SetValue("DrawLayer", value); }
        }

        /// <summary>
        /// Gets or sets the minimum value of the status bar.
        /// </summary>
        [XmlAttribute("minValue")]
        [Category("Behavior")]
        [Description("Defines the lowest numeric value a control will accept.")]
        public float MinValue
        {
            get { return Properties.GetValue<float>("MinValue"); }
            set { Properties.SetValue("MinValue", value); }
        }

        /// <summary>
        /// Gets or sets a value indicating whether the <see cref="MinValue"/> is specified. This property is not serialized.
        /// </summary>
        [XmlIgnore]
        [Browsable(false)]
        public bool MinValueSpecified { get; set; }

        /// <summary>
        /// Gets or sets the maximum value of the status bar.
        /// </summary>
        [XmlAttribute("maxValue")]
        [Category("Behavior")]
        [Description("Defines the highest numeric value a control will accept.")]
        public float MaxValue
        {
            get { return Properties.GetValue<float>("MaxValue"); }
            set { Properties.SetValue("MaxValue", value); }
        }

        /// <summary>
        /// Gets or sets a value indicating whether the <see cref="MaxValue"/> is specified. This property is not serialized.
        /// </summary>
        [XmlIgnore]
        [Browsable(false)]
        public bool MaxValueSpecified { get; set; }

        /// <summary>
        /// Gets or sets the default value of the status bar.
        /// </summary>
        [XmlAttribute("defaultValue")]
        [Category("Behavior")]
        [Description("The DefaultValue property specifies the default value for the control.")]
        public float DefaultValue
        {
            get { return Properties.GetValue<float>("DefaultValue"); }
            set { Properties.SetValue("DefaultValue", value); }
        }

        /// <summary>
        /// Gets or sets a value indicating whether the <see cref="DefaultValue"/> is specified. This property is not serialized.
        /// </summary>
        [XmlIgnore]
        [Browsable(false)]
        public bool DefaultValueSpecified { get; set; }

        /// <summary>
        /// Gets or sets the orientation of the status bar. The default value is horizontal.
        /// </summary>
        [XmlAttribute("orientation")]
        [DefaultValue(Orientation.Horizontal)]
        [Category("Behavior")]
        [Description("Defines the orientation of the status bar. The default value is horizontal.")]
        public Orientation Orientation
        {
            get { return Properties.GetValue<Orientation>("Orientation"); }
            set { Properties.SetValue("Orientation", value); }
        }

        /// <summary>
        /// Gets or sets the texture used for the status bar.
        /// </summary>
        [XmlElement]
        [Category("Appearance")]
        [Description("Defines the texture used for the status bar.")]
        public Texture BarTexture
        {
            get { return Properties.GetValue<Texture>("BarTexture"); }
            set { Properties.SetValue("BarTexture", value); }
        }

        /// <summary>
        /// Gets or sets the color used for the status bar.
        /// </summary>
        [XmlElement]
        [EditorBrowsable]
        [Category("Appearance")]
        [Description("Defines the color used for the status bar.")]
        [Color]
        public Color BarColor
        {
            get { return Properties.GetValue<Color>("BarColor"); }
            set { Properties.SetValue("BarColor", value); }
        }

        /// <summary>
        /// Gets a value indicating whether a serialization object is complete and can be rendered.
        /// </summary>
        public override bool IsComplete
        {
            get
            {
                return this.Layers.SelectMany(layer => layer.Items)
                           .OfType<Texture>()
                           .Count() > 0;
            }
        }
    }
}