using System;
using System.ComponentModel;

namespace Microsoft.WowAddonStudio.FrameXml.Serialization.TypeConverters
{
	/// <summary>
	/// Provides a unified way of converting types of values to other types,
	/// as well as for accessing standard values and subproperties.
	/// </summary>
	public class NullTypeConverter : TypeConverter
	{
		/// <summary>
		/// Returns whether this converter can convert an object of the given type to the type of this converter, using the specified context.
		/// </summary>
		/// <returns>
		/// true if this converter can perform the conversion; otherwise, false.
		/// </returns>
		/// <param name="context">An <see cref="T:System.ComponentModel.ITypeDescriptorContext"></see> that provides a format context. </param>
		/// <param name="sourceType">A <see cref="T:System.Type"></see> that represents the type you want to convert from. </param>
		public override bool CanConvertFrom(ITypeDescriptorContext context, Type sourceType)
		{
			return false;
		}

		/// <summary>
		/// Returns whether this converter can convert the object to the specified type, using the specified context.
		/// </summary>
		/// <returns>
		/// true if this converter can perform the conversion; otherwise, false.
		/// </returns>
		/// <param name="context">An <see cref="T:System.ComponentModel.ITypeDescriptorContext"></see> that provides a format context. </param>
		/// <param name="destinationType">A <see cref="T:System.Type"></see> that represents the type you want to convert to. </param>
		public override bool CanConvertTo(ITypeDescriptorContext context, Type destinationType)
		{
			return false;
		}
	}
}