﻿using System;
using System.Linq;

using Microsoft.WowAddonStudio.Lua.AST;
using Microsoft.WowAddonStudio.Lua.Model;
using Microsoft.WowAddonStudio.Lua.Support;

namespace Microsoft.WowAddonStudio.Lua.Adapters
{
    /// <summary>
    /// Adds documentation for the <see cref="FunctionDeclaration"/> nodes in the AST.
    /// </summary>
    public class FunctionDeclarationDocumentationAdapter : DocumentationAdapter<FunctionDeclaration>
    {
        public override void AddDocumentation(DocumentationScope documentationScope, FunctionDeclaration node)
        {
            base.AddDocumentation(documentationScope, node);

            // Create declaration for the function itself and add it to correct scope
            var declaration = DeclarationFactory.CreateDeclaration(node, node.IsLocal);

            // Add the declaration to the documentation scope
            documentationScope.Add(declaration);

            // Create declarations for the parameters and add it to the block's scope
            var bodyScope = node.InnerScope as DocumentationScope;
            if (bodyScope != null)
            {
                var parameters = node.GetParameterIdentifiers().Select(identifier => DeclarationFactory.CreateDeclaration<Parameter>(identifier, true));
                bodyScope.AddRange(parameters.Cast<Declaration>());
            }
        }
    }
}