﻿using System.Collections.Generic;

namespace Microsoft.WowAddonStudio.Project.Import
{
    /// <summary>
    /// Defines an interface that imports addon items from location specified by user.
    /// </summary>
    public interface IImportPlugin
    {
        /// <summary>
        /// Gets the user interface element used to pick the uri to import from.
        /// </summary>
        /// <returns>An instance of the <see cref="ImportWizardPane"/> class.</returns>
        ImportWizardPane GetImportWizardPane();

        /// <summary>
        /// Imports the addon at the given uri.
        /// </summary>
        /// <param name="importSettings">The import settings.</param>
        /// <returns>An enumerable collection of <see cref="IAddonItem"/>s.</returns>
        IEnumerable<IAddonItem> Import(object importSettings);


		/// <summary>
		/// Gets the name of the root directory. Used when the import process
		/// (<see cref="ImportProcess"/>) creates subdirectory under the target
		/// folder based on the value of this property.
		/// </summary>
		/// <value>The name of the root directory.</value>
		string RootDirectoryName { get; }
    }
}
