using System;
using System.IO;
using System.Runtime.InteropServices;
using System.Windows.Forms;
using Microsoft.VisualStudio.Shell;

namespace Microsoft.WowAddonStudio.Project.OptionPages
{
    [GuidAttribute(GuidStrings.WowBenchGeneralOptionsDialogPage)]
    public class WowBenchGeneralOptionsDialogPage : DialogPage
    {
        private readonly WowBenchGeneralOptionsControl control = new WowBenchGeneralOptionsControl();
        
        /// <summary>
        /// Gets the Window control.
        /// </summary>
        /// <value></value>
        /// <!-- Failed to insert some or all of included XML -->
        /// <include file="doc\DialogPage.uex" path="docs/doc[@for=&quot;DialogPage&quot;.Window]"/>
        /// <devdoc>
        /// The window this dialog page will use for its UI.
        /// This window handle must be constant, so if you are
        /// returning a Windows Forms control you must make sure
        /// it does not recreate its handle.  If the window object
        /// implements IComponent it will be sited by the
        /// dialog page so it can get access to global services.
        /// </devdoc>
        protected override IWin32Window Window
        {
            get { return control; }
        }

        /// <summary>
        /// WoWBench intstallation folder path.
        /// </summary>
        /// <value>The WoW bench path.</value>
        public string CustomWowBenchPath
        {
            get { return control.WowBenchPath; }
            set { control.WowBenchPath = value; }
        }

        /// <summary>
        /// </summary>
        /// <param name="e"></param>
        /// <!-- Failed to insert some or all of included XML -->
        /// <include file="doc\DialogPage.uex" path="docs/doc[@for=&quot;DialogPage.OnApply&quot;]"/>
        /// <devdoc>
        /// This method is called when VS wants to save the user's
        /// changes then the dialog is dismissed.
        /// </devdoc>
        protected override void OnApply(PageApplyEventArgs e)
        {
            if (CustomWowBenchPath != null && !Directory.Exists(CustomWowBenchPath))
            {
                e.ApplyBehavior = ApplyKind.Cancel;
                MessageBox.Show(Window, Resources.GetString(Resources.InvalidWoWBenchIntstallationPathMessag),
                                Resources.AddOnStudioforWorldofWarcraft, MessageBoxButtons.OK, MessageBoxIcon.Warning);
            }
            else
            {
                base.OnApply(e);
            }
        }
    }
}