﻿using System;
using Microsoft.WowAddonStudio.Project.OptionPages;
using Microsoft.WowAddonStudio.Services;

namespace Microsoft.WowAddonStudio.Project.Services
{
    public class AddonStudioSettings : IAddonStudioSettings
    {
        private readonly WowProjectPackage projectPackage;

        /// <summary>
        /// Initializes a new instance of the <see cref="AddonStudioSettings"/> class.
        /// </summary>
        /// <param name="projectPackage">The project package.</param>
        public AddonStudioSettings(WowProjectPackage projectPackage)
        {
            if (projectPackage == null)
                throw new ArgumentNullException("projectPackage");

            this.projectPackage = projectPackage;
        }

        /// <summary>
        /// Occurs when the background image file changes.
        /// </summary>
        public event EventHandler BackgroundImageFileChanged
        {
            add { projectPackage.GetDialogPage<FrameXmlDesignerGeneralOptionsDialogPage>().BackgroundImageFileChanged += value; }
            remove { projectPackage.GetDialogPage<FrameXmlDesignerGeneralOptionsDialogPage>().BackgroundImageFileChanged -= value; }
        }

        /// <summary>
        /// Gets the BackgroundImageFile setting.
        /// </summary>
        public string BackgroundImageFile
        {
            get { return projectPackage.GetDialogPage<FrameXmlDesignerGeneralOptionsDialogPage>().BackgroundImageFile; }
        }

        /// <summary>
        /// Gets the WoWBench folder setting.
        /// </summary>
        public string CustomWowBenchPath
        {
            get { return projectPackage.GetDialogPage<WowBenchGeneralOptionsDialogPage>().CustomWowBenchPath; }
            set { projectPackage.GetDialogPage<WowBenchGeneralOptionsDialogPage>().CustomWowBenchPath = value; }
        }

        /// <summary>
        /// Gets the WoW base path folder setting.
        /// </summary>
        public string CustomDeploymentLocation
        {
            get { return projectPackage.GetDialogPage<AddOnStudioGeneralOptionsDialogPage>().CustomDeploymentLocation; }
        }

        /// <summary>
        /// Gets or sets a value indicating whether [open in editor].
        /// </summary>
        /// <value><c>true</c> if [open in editor]; otherwise, <c>false</c>.</value>
        public bool OpenInEditor
        {
            get { return projectPackage.GetDialogPage<AddOnStudioGeneralOptionsDialogPage>().OpenInEditor; }
            set { projectPackage.GetDialogPage<AddOnStudioGeneralOptionsDialogPage>().OpenInEditor = value; }
        }

    }
}